
/*!
 Copyright 2009 Sun Microsystems, Inc.
 
 @author
 jak
 
 @class
 WBTabView
 
 @abstract
 WBTabView implements a customized look for tab views, while trying to be semantically compatible with NSTabView.
 WBTabView is the base class of several customized classes each implementing a different look.
 
 @discussion
 An WBTabView inherits from NSTabView. The method -doCustomize does the following:
 
 1. Adds a tabless NSTabView as subview inside self, and moves any existing tab view item to it.
 2. Creates a view mTabRowView in which it draws its customized tabs.
 
 The custom tabs are build with a hierarchy of core animation layers.
 */



#import "WBTabView.h"
#import "ResponderLayer.h"
#import "WBTabItem.h"
#import "CGColorUtilities.h"

#import <Quartz/Quartz.h>

// Import the runtime stuff that enable swizzling.
#import <objc/objc-class.h>


#define TAB_ITEM_SPACING (4)
#define TAB_ITEM_SMALL_SPACING (0)




// Unspeakably neat Obj-C swizzling!
// We want to call the tabView's delegate to notify it that the NSTabViewItem has changed its label.
// Since a normal NSTabViewItem in Cocoa does not provide any callack to its delegate, we need to do that ourselves.
// We do it by:
// 1. Implementing a "better" version of the method -setLabel: with the name -alternateSetLabel:
// 2. Later, we exchange the new and rhe original methods. See method_exchangeImplementations().
@interface NSTabViewItem (WBTabView_swizzling_additions)
- (void) alternateSetLabel: (NSString*) newLabel;
@end

@implementation NSTabViewItem (WBTabView_swizzling_additions)
- (void) alternateSetLabel: (NSString*) newLabel;
{
  // Calling the same method self does not lead to an infinite recursion, because the
  // method name -alternateSetLabel: now "points" to the original -setLabel:
  // Unspeakably neat, no!?
  [self alternateSetLabel: newLabel];
  
  // Then call the tabview's delegate to notify.
  [(WBTabView*)[[self tabView] superview] updateLabelForTabViewItem: [self identifier]];
}
@end






@interface WBRightClickThroughView : NSView
@end

@implementation WBRightClickThroughView

- (void) rightMouseDown: (NSEvent*) event;
{
  [[self superview] rightMouseDown: event];
}
  
@end



@implementation WBTabView



- (BOOL) isFlipped;
{
	return NO;
}



- (CGFloat) tabAreaHeight;
{
	return 26.0;
}



- (NSSize) contentSize;
{
	NSSize size = NSZeroSize;
	
	if (mTabView != nil) {
		size = [mTabView contentRect].size;
	}
	
	return size;
}



- (CGColorRef) tabRowActiveBackgroundColorCreate;
{
	return WB_CGColorCreateCalibratedRGB(0.52, 0.52, 0.52, 1);
}



- (CGColorRef) tabRowInactiveBackgroundColorCreate;
{
	return WB_CGColorCreateCalibratedRGB(0.8, 0.8, 0.8, 1);
}



- (WBTabItem*) tabItemWithIdentifier: (id) identifier
															 label: (NSString*) label;
{
	WBTabItem* item = [WBTabItem tabItemWithIdentifier: identifier
																							 label: label
																					 direction: mTabDirection
																					 placement: mTabPlacement
																								size: mTabSize
																						 hasIcon: (mTabSize == WBTabSizeLarge)
																						canClose: (mTabSize == WBTabSizeLarge)];
	
	return item;
}



//- (void) setAllowsTabReordering: (BOOL) yn;
//{
//	mAllowsTabReordering = yn;
//}



#pragma mark Private API called from public API



- (void) layoutTabItemsDisregardingTabItem: (WBTabItem*) tabItemToDisregard;
{
	CGSize tabSize = [(WBTabItem*)[mTabItems lastObject] frame].size;
	CGFloat tabHeightHalf = (tabSize.height / 2);
	
	CGFloat horizon;
	if (mTabPlacement == WBTabPlacementTop) {
		if (mTabDirection == WBTabDirectionUp) {
			// Tabs sticking up from the tab view.
			horizon = 0;
		}
		else {
			// Tabs hanging down from the ceiling pointing towards the tabview,
			// like those hanging from the window's toolbar.
			horizon = [mTabRowLayer frame].size.height;
		}
	}
	else {
		if (mTabDirection == WBTabDirectionUp) {
			NSAssert(NO, @"Strange tab configuration.");
		}
		else {
			horizon = [mTabRowLayer frame].size.height;
		}
	}
	CGFloat tabBottom = horizon - tabHeightHalf;
	
	CGFloat lastEdge = TAB_ITEM_SPACING;
	for (WBTabItem* item in mTabItems) {
		[item updateAppearance];
		CGRect itemFrame = [item frame];
		if (item != tabItemToDisregard) {
			CGFloat x = lastEdge + (mTabSize == WBTabSizeLarge ? TAB_ITEM_SPACING : TAB_ITEM_SMALL_SPACING);
			CGRect r = CGRectMake(x, tabBottom, itemFrame.size.width, itemFrame.size.height);
			[item setFrame: r];
		}
    
		lastEdge += itemFrame.size.width + (mTabSize == WBTabSizeSmall ? TAB_ITEM_SMALL_SPACING : TAB_ITEM_SPACING);
	}
}



// Delayed call after TabView is resized.
- (void) layoutTabItemsQuickly;
{
  [[NSAnimationContext currentContext] setDuration: 0.0];
	[self layoutTabItemsDisregardingTabItem: nil];
}



- (void) addTabItemWithIdentifier: (id) identifier
														label: (NSString*) label;
{
	WBTabItem* item = [self tabItemWithIdentifier: identifier
																					label: label];
	if (mTabPlacement == WBTabPlacementTop) {
		[item setAutoresizingMask: (kCALayerMaxXMargin | kCALayerMinYMargin)];
	}
	else {
		[item setAutoresizingMask: (kCALayerMaxXMargin | kCALayerMaxYMargin)];
	}
	
	[item setDelegate: self];
	
	if (mTabItems == nil) {
		mTabItems = [NSMutableArray new];
	}
	[mTabItems addObject: item];
	[mTabRowLayer addSublayer: item];
}



- (void) removeTabViewWithIdentifier: (id) identifier;
{
	for (WBTabItem* item in mTabItems) {
		if ([[item identifier] isEqual: identifier]) {
			[[NSAnimationContext currentContext] setDuration: 0.1];
			
			int index = [mTabItems indexOfObject: item];
			[[item retain] autorelease];
			[item removeFromSuperlayer];
			[mTabItems removeObject: item];
			
			if (mSelectedTab == item) {
				mSelectedTab = nil;
				WBTabItem* newTab = nil;
				if (index == [mTabItems count]) {
					newTab = [mTabItems lastObject];
				}
				else {
					newTab = [mTabItems objectAtIndex: index];
				}
				
				[self selectTabViewItemWithIdentifier: [newTab identifier]];
			}
			
			for (NSTabViewItem* tabViewItem in [mTabView tabViewItems]) {
				if ([tabViewItem identifier] == identifier) {
					[mTabView removeTabViewItem: tabViewItem];
				}
			}
			
			break;
		}
	}
}



#pragma mark Overriding public NSTabView API




- (void) addTabViewItem: (NSTabViewItem*) tabViewItem;
{
	if (mTabView != nil) {
		[mTabView addTabViewItem: tabViewItem];
		
		id identifier = [tabViewItem identifier];
		NSString* label = [tabViewItem label];
		[self addTabItemWithIdentifier: identifier
														 label: label];
		[self layoutTabItemsDisregardingTabItem: nil];
		
		if (mDoneCustomizing) {
			if ([[self delegate] respondsToSelector: @selector(tabViewDidChangeNumberOfTabViewItems:)]) {
				[[self delegate] tabViewDidChangeNumberOfTabViewItems: self];
			}
		}
		
		if ([mTabItems count] == 1) {
			[self selectLastTabViewItem: self];
		}
	}
	else {
		[super addTabViewItem: tabViewItem];
	}
}



- (void) removeTabViewItem: (NSTabViewItem*) tabViewItem;
{
	if (mTabView != nil) {
		id identifier = [tabViewItem identifier];
		[self removeTabViewWithIdentifier: identifier];
		[self layoutTabItemsDisregardingTabItem: nil];
		
		if (mDoneCustomizing) {
			if ([[self delegate] respondsToSelector: @selector(tabViewDidChangeNumberOfTabViewItems:)]) {
				[[self delegate] tabViewDidChangeNumberOfTabViewItems: self];
			}
		}
	}
	else if (mDoneCustomizing) {
		[super removeTabViewItem: tabViewItem];
	}
}



- (NSArray*) tabViewItems;
{
	if (mTabView != nil)
		return [mTabView tabViewItems];
	else
		return [super tabViewItems];
}



- (NSTabViewItem*) tabViewItemAtIndex: (NSInteger) index;
{
	if (mTabView != nil)
		return [mTabView tabViewItemAtIndex: index];
	else
		return [super tabViewItemAtIndex: index];
}



- (NSInteger) indexOfTabViewItemWithIdentifier: (id) identifier;
{
	if (mTabView != nil)
		return [mTabView indexOfTabViewItemWithIdentifier: identifier];
	else
		return [super indexOfTabViewItemWithIdentifier: identifier];
}



- (NSInteger) indexOfTabViewItem: (NSTabViewItem*) tabViewItem;
{
	return [self indexOfTabViewItemWithIdentifier: [tabViewItem identifier]];
}



- (void) selectTabViewItemWithIdentifier: (id) identifier;
{
	if (mTabView != nil) {
		NSTabViewItem* tabViewItem = nil;
		for (NSTabViewItem* item in [mTabView tabViewItems]) {
			if ([[item identifier] isEqual: identifier]) {
				tabViewItem = item;
				break;
			}
		}
		
		if (tabViewItem != nil) {
			for (WBTabItem* item in mTabItems) {
                                if ([[item identifier] isEqual: identifier]) {
					BOOL shouldSelect = YES;
					if (mDoneCustomizing) {
						if ([[self delegate] respondsToSelector: @selector(tabView:shouldSelectTabViewItem:)]) {
							shouldSelect = [[self delegate] tabView: self
															shouldSelectTabViewItem: tabViewItem];
						}
					}
					if (shouldSelect) {
						if (mDoneCustomizing) {
							if ([[self delegate] respondsToSelector: @selector(tabView:willSelectTabViewItem:)]) {
								[[self delegate] tabView: self
									 willSelectTabViewItem: tabViewItem];
							}
						}
						
						id old = mSelectedTab;
						id new = item;
						mSelectedTab = new;
						[old setState: NSOffState];
						[new setState: NSOnState];
						[mTabView selectTabViewItemWithIdentifier: identifier];
						
						if (mDoneCustomizing) {
							if ([[self delegate] respondsToSelector: @selector(tabView:didSelectTabViewItem:)]) {
								[[self delegate] tabView: self
										didSelectTabViewItem: tabViewItem];
							}
						}
					}
					break;
				}
			}
		}
	}
	else {
		[super selectTabViewItemWithIdentifier: identifier];
	}
}



- (void) selectFirstTabViewItem: (id) sender;
{
	if ( (mTabView != nil) && ([mTabItems count] > 0) ) {
		id identifier = [[mTabItems objectAtIndex: 0] identifier];
		[self selectTabViewItemWithIdentifier: identifier];
	}
	else {
		[super selectFirstTabViewItem: sender];
	}
}



- (void) selectLastTabViewItem: (id) sender;
{
	if (mTabView != nil) {
		id identifier = [[mTabItems lastObject] identifier];
		[self selectTabViewItemWithIdentifier: identifier];
	}
	else {
		[super selectLastTabViewItem: sender];
	}
}



- (NSTabViewItem*) selectedTabViewItem;
{
	if (mTabView != nil)
		return [mTabView selectedTabViewItem];
	else
		return [super selectedTabViewItem];
}



- (NSInteger) numberOfTabViewItems;
{
	if (mTabView != nil)
		return [mTabView numberOfTabViewItems];
	else
		return [super numberOfTabViewItems];
}



- (NSRect) contentRect
{
	if (mTabView != nil)
		return [mTabView contentRect];
	else
		return [super contentRect];
}



- (void) selectNextTabViewItem: (id) sender;
{
	NSTabViewItem* item = [self selectedTabViewItem];
	NSInteger index = [self indexOfTabViewItem: item];
	index ++;
	if (index < [mTabItems count]) {
		id identifier = [[mTabItems objectAtIndex: index] identifier];
		[self selectTabViewItemWithIdentifier: identifier];
	}
}



- (void) insertTabViewItem: (NSTabViewItem*) tabViewItem
									 atIndex: (NSInteger) index;
{
	NSLog(@"***************************************");
	NSAssert(NO, @"Not implemented: insertTabViewItem:atIndex:");
}



#pragma mark -



- (void) setFrame: (NSRect) frame;
{
	[super setFrame: frame];

	[self performSelector: @selector(layoutTabItemsQuickly)
             withObject: nil
             afterDelay: 0];
}



#pragma mark TabItemDelegateProtocol methods



- (void) selectTab: (WBTabItem*) sender;
{
	[[NSAnimationContext currentContext] setDuration: 0.0];
	[self selectTabViewItemWithIdentifier: [sender identifier]];
}



- (void) closeTab: (WBTabItem*) sender;
{
  [[sender retain] autorelease];
  
	if (mDoneCustomizing) {
		if ([[self delegate] respondsToSelector: @selector(tabView:willCloseTabViewItem:)]) {
			for (NSTabViewItem* item in [mTabView tabViewItems]) {
				if ([[item identifier] isEqual: [sender identifier]]) {
					[(id<WBTabViewDelegateProtocol>)[self delegate] tabView: self
					 willCloseTabViewItem: item];
				}
			}
		}
	}
	
	for (NSTabViewItem* item in [mTabView tabViewItems]) {
		if ([item identifier] == [sender identifier]) {
			[self removeTabViewItem: item];
			break;
		}
	}
}



- (CGRect) tabItem: (WBTabItem*) sender
		draggedToFrame: (CGRect) r;
{
	if (r.origin.x < TAB_ITEM_SPACING) {
		r.origin.x = TAB_ITEM_SPACING;
	}
	
	NSMutableSet* tabsToDisregard = [NSMutableSet set];
	[tabsToDisregard addObject: sender];
	BOOL didMoveOne;
	do {
		didMoveOne = NO;
		NSInteger ix = [mTabItems indexOfObject: sender];
		int i, c = [mTabItems count];
		for (i = 0; i < c; i++) {
			WBTabItem* peer = [mTabItems objectAtIndex: i];
			if (! [tabsToDisregard containsObject: peer]) {
				CGRect peerFrame = [peer frame];
				CGFloat peerMid = CGRectGetMinX(peerFrame) + (peerFrame.size.width / 2);
				if (i > ix) {
					if (CGRectGetMaxX(r) > peerMid) {
						[mTabItems removeObject: peer];
						[mTabItems insertObject: peer
														atIndex: ix];
						didMoveOne = YES;
						[tabsToDisregard addObject: peer];
						break;
					}
				}
				else {
					if (CGRectGetMinX(r) < peerMid) {
						[mTabItems removeObject: peer];
						[mTabItems insertObject: peer
														atIndex: ix];
						didMoveOne = YES;
						[tabsToDisregard addObject: peer];
						break;
					}
				}
			}
		}
	} while (didMoveOne);
	
	[[NSAnimationContext currentContext] setDuration: 0.1];
	[self layoutTabItemsDisregardingTabItem: sender];
	
	return r;
}



#pragma mark User interaction



- (void) mouseDown: (NSEvent*) event;
{
	NSAssert( (mMouseDownLayer == nil), @"mMouseDownLayer was not nil.");

  if ( ([event modifierFlags] & NSControlKeyMask) != 0 ) {
    [self rightMouseDown: event];
  }
  else {
    NSPoint loc = [mTabRowView convertPoint: [event locationInWindow]
                                   fromView: nil];
    CGPoint cgp = NSPointToCGPoint(loc);
    mMouseDownLayer = [mTabRowLayer mouseDownAtPoint: cgp];
    
    if ([event clickCount] > 1) {
      // This was a double click.
      if ([[self delegate] respondsToSelector: @selector(tabViewItemDidReceiveDoubleClick:)]) {
        [[self delegate] performSelector: @selector(tabViewItemDidReceiveDoubleClick:)
                              withObject: [self selectedTabViewItem]];
      }
    }
    
    if (mMouseDownLayer == nil) {
      [[self nextResponder] mouseDown: event];
    }
  }
}



- (void) mouseDragged: (NSEvent*) event;
{
//	if ( mAllowsTabReordering && (mMouseDownLayer != nil) ) {
	if (mMouseDownLayer != nil) {
		NSPoint loc = [mTabRowView convertPoint: [event locationInWindow]
																	 fromView: nil];
		CGPoint cgp = NSPointToCGPoint(loc);
		cgp = [mTabRowLayer convertPoint: cgp
														 toLayer: mMouseDownLayer];
		[mMouseDownLayer mouseDraggedToPoint: cgp];
	}
	else {
		[[self nextResponder] mouseDragged: event];
	}
}



- (void) mouseUp: (NSEvent*) event;
{
	if (mMouseDownLayer != nil) {
		[mMouseDownLayer mouseUp];
		mMouseDownLayer = nil;
		
		[[NSAnimationContext currentContext] setDuration: 0.1];
		[self layoutTabItemsDisregardingTabItem: nil];
	}
	else {
		[[self nextResponder] mouseUp: event];
	}
}



- (NSMenu*) menuForEvent: (NSEvent*) theEvent;
{
  NSMenu* menu = nil;
  
  if ([[self delegate] respondsToSelector: @selector(tabView:menuForIdentifier:)]) {
    NSPoint loc = [mTabRowView convertPoint: [theEvent locationInWindow]
                                   fromView: nil];
    CGPoint cgp = NSPointToCGPoint(loc);
    ResponderLayer* item = [mTabRowLayer responderLayerAtPoint: cgp];
    if ([item isKindOfClass: [WBTabItem class]]) {
      menu = [[self delegate] performSelector: @selector(tabView:menuForIdentifier:)
                                   withObject: self
                                   withObject: [(WBTabItem*)item identifier]];
    }
  }
  
	return menu;
}



#pragma mark -



- (void) handleDidBecomeMain: (id) aNotification;
{
	mEnabled = YES;
	[[NSAnimationContext currentContext] setDuration: 0];
	
	[self layoutTabItemsDisregardingTabItem: nil];
	
	for (WBTabItem* item in mTabItems) {
		[item setEnabled: YES];
	}
	
	CGColorRef tabRowBackgroundColor = [self tabRowActiveBackgroundColorCreate];
  if (tabRowBackgroundColor != nil) {
    [mTabRowLayer setBackgroundColor: tabRowBackgroundColor];
  }
	CGColorRelease(tabRowBackgroundColor);
}



- (void) handleDidResignMain: (id) aNotification;
{
	mEnabled = NO;
	[[NSAnimationContext currentContext] setDuration: 0];
	
	for (WBTabItem* item in mTabItems) {
		[item setEnabled: NO];
	}
	
	CGColorRef tabRowBackgroundColor = [self tabRowInactiveBackgroundColorCreate];
  if (tabRowBackgroundColor != nil) {
    [mTabRowLayer setBackgroundColor: tabRowBackgroundColor];
  }
	CGColorRelease(tabRowBackgroundColor);
}



#pragma mark Setup



- (void) updateLabelForTabViewItem: (id) identifier;
{
  for (NSTabViewItem *item in [self tabViewItems]) {
    if ([item identifier] == identifier) {
      NSString* label = [item label];
      for (WBTabItem* customTab in mTabItems) {
        if ([[customTab identifier] isEqual: identifier]) {
          [customTab setLabel: label];
        }
      }
      break;
    }
  }
  [self layoutTabItemsDisregardingTabItem: nil];
}



- (float) contentPadding;
{
  return 0;
}



- (void) makeInitialSwap;
{
	[self setTabViewType: NSNoTabsNoBorder];
	
	// Create the NSTabView to hold the actual views.
	{
		NSRect newFrame = [self frame];
		newFrame.origin.x += 7;
		newFrame.size.width -= 14;
		newFrame.size.height -= 16;
		if (mTabPlacement == WBTabPlacementTop) {
			newFrame.origin.y += 10;
		}
		else {
			newFrame.origin.y += 6;
		}
		[self setFrame: newFrame];
	}
	
	// Set up the NSTabView that will contain and display the actual content.
	NSRect contentFrame = [self bounds];
	contentFrame.size.height -= [self tabAreaHeight];
	if (mTabPlacement == WBTabPlacementBottom) {
		contentFrame.origin.y = [self tabAreaHeight];
	}
  
  float contentPadding = [self contentPadding];
  contentFrame.origin.x += contentPadding;
  contentFrame.size.width -= 2 * contentPadding;
  contentFrame.size.height -= contentPadding;
  
	NSTabView* theContainerTabView = [[[NSTabView alloc] initWithFrame: contentFrame] autorelease];
	[theContainerTabView setTabViewType: NSNoTabsNoBorder];
	[theContainerTabView setAutoresizingMask: (NSViewWidthSizable | NSViewHeightSizable)];
	
	// Set up the view that will contain the custom tabs.
	NSRect tabRowFrame = [self bounds];
	tabRowFrame.size.height = [self tabAreaHeight];
	if (mTabPlacement == WBTabPlacementTop) {
		tabRowFrame.origin.y = contentFrame.size.height;
	}
	mTabRowView = [[[WBRightClickThroughView alloc] initWithFrame: tabRowFrame] autorelease];
	if (mTabPlacement == WBTabPlacementTop) {
		[mTabRowView setAutoresizingMask: (NSViewWidthSizable | NSViewMinYMargin)];
	}
	else {
		[mTabRowView setAutoresizingMask: (NSViewWidthSizable | NSViewMaxYMargin)];
	}
	
	[self setAutoresizesSubviews: YES];
	
	NSArray* tabViewItems = [self tabViewItems];
	for (NSTabViewItem* item in tabViewItems) {
		[theContainerTabView addTabViewItem: item];
	}
	
	mTabView = theContainerTabView;
	
	[self addSubview: mTabRowView];
	mTabRowLayer = [ResponderLayer layer];
	CGColorRef tabRowBackgroundColor = [self tabRowActiveBackgroundColorCreate];
  if (tabRowBackgroundColor != nil) {
    [mTabRowLayer setBackgroundColor: tabRowBackgroundColor];
  }
	CGColorRelease(tabRowBackgroundColor);
	[mTabRowLayer setZPosition: -5];
	
	[mTabRowView setWantsLayer: YES];
	[mTabRowView setLayer: mTabRowLayer];
	
	[self addSubview: mTabView];
}



// Create layer to cast a shadow from the top of the view.
- (CALayer*) shadowLayer;
{
	CALayer* shadowLayer = [CALayer layer];
	
	CGColorRef bc = WB_CGColorCreateCalibratedRGB(1, 0.8, 0.8, 1);
	[shadowLayer setBackgroundColor: bc];
	CGColorRelease(bc);
	
	[shadowLayer setBorderWidth: 1];
	CGColorRef c = WB_CGColorCreateCalibratedRGB(0.3, 0.3, 0.3, 0.7);
	[shadowLayer setBorderColor: c];
	CGColorRelease(c);
	
	[shadowLayer setShadowOpacity: 0.3];
	[shadowLayer setShadowOffset: CGSizeMake(0, -1)];
	CGRect r = [mTabRowLayer frame];
	if (mTabPlacement == WBTabPlacementTop) {
		r.origin.y = r.size.height - 1;
		[shadowLayer setAutoresizingMask: (kCALayerWidthSizable | kCALayerMinYMargin)];
	}
	else {
		r.origin.y = [self tabAreaHeight];
		
		[shadowLayer setAutoresizingMask: (kCALayerWidthSizable | kCALayerMaxYMargin)];
	}
	r.size.height = 10;
	r.origin.x = -10;
	r.size.width += 20;
	[shadowLayer setFrame: r];
	[shadowLayer setZPosition: -2];
	
	return shadowLayer;
}



- (CALayer*) plaqueLayer;
{
  return nil;
}



- (CALayer*) lineLayer;
{
  // Create a line between non-selected tabs and the tabviewcontants.
  CALayer* lineLayer = [CALayer layer];;
  
  [lineLayer setBorderWidth: 1];
  CGColorRef c = WB_CGColorCreateCalibratedRGB(0.3, 0.3, 0.3, 0.7);
  [lineLayer setBorderColor: c];
  CGColorRelease(c);
  
  CGRect r = CGRectZero;
  r.origin.x = 0;
  if (mTabDirection == WBTabDirectionUp) {
    r.origin.y = 0;
    [lineLayer setAutoresizingMask: (kCALayerWidthSizable | kCALayerMaxYMargin)];
  }
  else {
    r.origin.y = [mTabRowLayer frame].size.height - 1;
    [lineLayer setAutoresizingMask: (kCALayerWidthSizable | kCALayerMinYMargin)];
  }
  
  r.size.width = [mTabRowLayer frame].size.width;
  r.size.height = 1;
  [lineLayer setFrame: r];
  
  [lineLayer setZPosition: -2];
  [mTabRowLayer addSublayer: lineLayer];
  
  return lineLayer;
}



- (void) createTabRow;
{
		CALayer* shadowLayer = [self shadowLayer];
		[mTabRowLayer addSublayer: shadowLayer];
	
//	{
//		CALayer* debugLayer = [CALayer layer];
//		
//		[debugLayer setBorderWidth: 1];
//		CGColorRef c = WB_CGColorCreateCalibratedRGB(1, 0.3, 0.3, 1);
//		[debugLayer setBorderColor: c];
//		CGColorRelease(c);
//		
//		CGRect r = [[mTabView layer] bounds];
//		[debugLayer setFrame: r];
//		[debugLayer setZPosition: 100];
//		[debugLayer setAutoresizingMask: (kCALayerWidthSizable | kCALayerHeightSizable)];
//		[[mTabView layer] addSublayer: debugLayer];
//	}
//	{
//		CALayer* debugLayer = [CALayer layer];
//		
//		[debugLayer setBorderWidth: 1];
//		CGColorRef c = WB_CGColorCreateCalibratedRGB(0.3, 0.9, 0.3, 1);
//		[debugLayer setBorderColor: c];
//		CGColorRelease(c);
//		
//		CGRect r = [mTabRowLayer bounds];
//		[debugLayer setFrame: r];
//		[debugLayer setZPosition: 100];
//		[debugLayer setAutoresizingMask: (kCALayerWidthSizable | kCALayerHeightSizable)];
//		[mTabRowLayer addSublayer: debugLayer];
//	}
	

  CALayer* plaqueLayer = [self plaqueLayer];
  [mTabRowLayer addSublayer: plaqueLayer];
  
  CALayer* lineLayer = [self lineLayer];
  [mTabRowLayer addSublayer: lineLayer];
  
	// Populate with tabs.
	{
		NSArray* tabViewItems = [mTabView tabViewItems];
		for (NSTabViewItem* item in tabViewItems) {
			id identifier = [item identifier];
			NSString* label = [item label];
			[self addTabItemWithIdentifier: identifier
															 label: label];
		}
		
		[self layoutTabItemsDisregardingTabItem: nil];
	}
}



- (void) doCustomize;
{
	if (mTabPlacement == WBTabPlacementUndefined) {
		NSLog(@"* Tab placement undefined. Using WBTabPlacementTop.");
		mTabPlacement = WBTabPlacementTop;
	}
	
	if (mTabDirection == WBTabDirectionUndefined) {
		NSLog(@"* Tab direction undefined. Using WBTabDirectionUp.");
		mTabDirection = WBTabDirectionUp;
	}
	
	if (mTabSize == WBTabSizeUndefined) {
		NSLog(@"* Tab size undefined. Using WBTabSizeLarge.");
		mTabSize = WBTabSizeLarge;
	}
	
	// Remember which tab was selected in IB.
	NSTabViewItem* selectedTabViewItem = [self selectedTabViewItem];
	id selectedTabIdentifier = [selectedTabViewItem identifier];
	
	// Create the new tab view layer hierarchy.
	[self makeInitialSwap];
	
	// Create our own custom tabs.
	[self createTabRow];
	
	// Select the tab that was selected in IB.
	if ( (selectedTabIdentifier == nil) && ([mTabItems count] > 0) ) {
		selectedTabIdentifier = [[mTabItems objectAtIndex: 0] identifier];
	}
	[self selectTabViewItemWithIdentifier: selectedTabIdentifier];
	
  // Unspeakably neat Obj-C swizzling!
  // Substitute -setLabel: method for NSTabVIewItem so that it notifies us so we
  // can update the corresponding custom tab.
  Method originalMethod = class_getInstanceMethod([NSTabViewItem class], @selector(setLabel:));
  Method alternateMethod = class_getInstanceMethod([NSTabViewItem class], @selector(alternateSetLabel:));
  method_exchangeImplementations(originalMethod, alternateMethod);
  
	// Set up notifications.
	NSNotificationCenter* dc = [NSNotificationCenter defaultCenter];
	[dc addObserver: self
				 selector: @selector(handleDidBecomeMain:)
						 name: NSWindowDidBecomeMainNotification
					 object: [self window]];
	[dc addObserver: self
				 selector: @selector(handleDidResignMain:)
						 name: NSWindowDidResignMainNotification
					 object: [self window]];
	
	[self layoutTabItemsDisregardingTabItem: nil];
	[self performSelector: @selector(layoutTabItemsDisregardingTabItem:)
						 withObject: nil
						 afterDelay: 0];

	mDoneCustomizing = YES;
}



#pragma mark Create and Destroy



- (void) awakeFromNib;
{
//	mAllowsTabReordering = YES;
	[self doCustomize];
}



- (void) dealloc
{
	NSNotificationCenter* dc = [NSNotificationCenter defaultCenter];
	[dc removeObserver: self];
	
	[mTabItems release];
	
	CGColorRelease(mColorActiveSelected);
	CGColorRelease(mColorActiveNotSelected);
	CGColorRelease(mColorNotActiveSelected);
	CGColorRelease(mColorNotActiveNotSelected);
	
	[super dealloc];
}


@end


