#include "stdafx.h"
#include "mysql_relationship_editor.h"


using namespace bec;


RelationshipEditorBE::RelationshipEditorBE(GRTManager *grtm, const workbench_physical_ConnectionRef &relationship)
: BaseEditor(grtm, relationship), _relationship(relationship)
{
}


void RelationshipEditorBE::set_caption(const std::string &caption)
{
  AutoUndoEdit undo(this, _relationship, "caption");
  _relationship->caption(caption);
  undo.end(_("Change Relationship Caption"));
}


std::string RelationshipEditorBE::get_caption()
{
  return _relationship->caption();
}


std::string RelationshipEditorBE::get_caption_long()
{
  return strfmt("'%s'  (%s)  '%s'", get_left_table_name().c_str(), get_caption().c_str(), get_right_table_name().c_str());
}


void RelationshipEditorBE::set_extra_caption(const std::string &caption)
{
  AutoUndoEdit undo(this, _relationship, "extraCaption");
  _relationship->extraCaption(caption);
  undo.end(_("Change Relationship 2nd Caption"));
}


std::string RelationshipEditorBE::get_extra_caption()
{
  return _relationship->extraCaption();
}


std::string RelationshipEditorBE::get_extra_caption_long()
{
  return strfmt("'%s' (%s) '%s'", get_right_table_name().c_str(), get_extra_caption().c_str(), get_left_table_name().c_str());
}


void RelationshipEditorBE::set_left_mandatory(bool flag)
{
  AutoUndoEdit undo(this);
  _relationship->foreignKey()->mandatory(flag?1:0);
  undo.end(_("Change Mandatory"));
}


bool RelationshipEditorBE::get_left_mandatory()
{
  return _relationship->foreignKey()->mandatory() != 0;
}


void RelationshipEditorBE::set_right_mandatory(bool flag)
{
  AutoUndoEdit undo(this);
  _relationship->foreignKey()->referencedMandatory(flag?1:0);
  undo.end(_("Change Referred Mandatory"));
}


bool RelationshipEditorBE::get_right_mandatory()
{
  return _relationship->foreignKey()->referencedMandatory() != 0;
}


void RelationshipEditorBE::set_to_many(bool flag)
{
  AutoUndoEdit undo(this);
  _relationship->foreignKey()->many(flag?1:0);
  undo.end(_("Change Relationship Cardinality"));
}


bool RelationshipEditorBE::get_to_many()
{
  return _relationship->foreignKey()->many() != 0;
}


void RelationshipEditorBE::set_comment(const std::string &comment)
{
  AutoUndoEdit undo(this, _relationship, "comment");
  _relationship->comment(comment);
  undo.end(_("Change Relationship Comment"));
}


std::string RelationshipEditorBE::get_comment()
{
  return _relationship->comment();
}



RelationshipEditorBE::VisibilityType RelationshipEditorBE::get_visibility()
{
  if (*_relationship->drawSplit() && *_relationship->visible())
    return Splitted;
  else if (!*_relationship->drawSplit() && *_relationship->visible())
    return Visible;
  else
    return Hidden;
}


void RelationshipEditorBE::set_visibility(VisibilityType type)
{
  if (get_visibility() == type) return;
  
  AutoUndoEdit undo(this);
  
  switch (type)
  {
    case Visible:
      _relationship->visible(1);
      _relationship->drawSplit(0);
      break;
    case Splitted:
      _relationship->visible(1);
      _relationship->drawSplit(1);
      break;
    case Hidden:
      _relationship->visible(0);
      break;
  }

  undo.end(_("Change Relationship Visibility"));
}


std::string RelationshipEditorBE::get_left_table_name()
{
  return *_relationship->foreignKey()->owner()->name();
}


std::string RelationshipEditorBE::get_right_table_name()
{
  return *_relationship->foreignKey()->referencedTable()->name();
}


std::string RelationshipEditorBE::get_left_table_info()
{
  std::string text;
  db_ForeignKeyRef fk(_relationship->foreignKey());
  for (size_t c= fk->columns().count(), i= 0; i < c; i++)
  {
    text.append(strfmt("%s: %s%s\n", fk->columns()[i]->name().c_str(), 
                       fk->columns()[i]->formattedType(0).c_str(),
                       fk->owner()->isPrimaryKeyColumn(fk->columns()[i])?" (PK)":""));
  }
  return text;
}


std::string RelationshipEditorBE::get_right_table_info()
{
  std::string text;  
  db_ForeignKeyRef fk(_relationship->foreignKey());
  for (size_t c= fk->columns().count(), i= 0; i < c; i++)
  {
    text.append(strfmt("%s: %s%s\n", fk->referencedColumns()[i]->name().c_str(), 
                       fk->referencedColumns()[i]->formattedType(0).c_str(),
                       fk->referencedTable()->isPrimaryKeyColumn(fk->referencedColumns()[i])?" (PK)":""));
  }
  return text;
}




std::string RelationshipEditorBE::get_left_table_fk()
{
  std::string text;
  db_ForeignKeyRef fk(_relationship->foreignKey());
  if (fk.is_valid())
    text= "Foreign Key: "+*fk->name();
  else
    text= "Foreign Key: NOT SET";
  return text;
}


void RelationshipEditorBE::open_editor_for_table(const db_TableRef &table)
{
  if (table.is_valid())
  {
    grt::BaseListRef args(get_grt(), grt::AnyType);
    args.ginsert(table);
    
    bec::GUIPluginFlags flags= bec::NoFlags;
    bec::PluginManager *pm= get_grt_manager()->get_plugin_manager();
    
    app_PluginRef plugin(pm->select_plugin_for_input("catalog/Editors", args));
    if (!plugin.is_valid())
      plugin= pm->select_plugin_for_input("model/Editors", args);
    
    if (plugin.is_valid())
      pm->open_gui_plugin(plugin, args, flags);
  }
}


void RelationshipEditorBE::open_editor_for_left_table()
{
  open_editor_for_table(_relationship->foreignKey()->owner());
}


void RelationshipEditorBE::open_editor_for_right_table()
{
  open_editor_for_table(_relationship->foreignKey()->referencedTable());
}


void RelationshipEditorBE::edit_left_table()
{
  open_editor_for_table(_relationship->foreignKey()->owner());
}


void RelationshipEditorBE::edit_right_table()
{
  open_editor_for_table(_relationship->foreignKey()->referencedTable());
}


void RelationshipEditorBE::invert_relationship()
{
}

